#include "autolist.h"
#include <iostream>
#include <fstream>

//#define DEBUG_AUTOLIST

/**
 * Create an empty AutoList
 */
AutoList::AutoList() {
}

/**
 * Create an autolist based on the given properties file
 */
AutoList::AutoList(string filename) {
    loadFile(filename);
}

/**
 * Destroy the AutoList
 */
AutoList::~AutoList() {
    directoryList.clear();
}

/**
 * Load the given file into the AutoList
 */
void AutoList::loadFile(string filename) {
    char line[1024];
    filebuf fb;
#ifdef DEBUG_AUTOLIST
    cout << "Opening " << filename << endl;
#endif
    // Open the given file
    fb.open(filename.c_str(), ios::in);
    istream in(&fb);
    directoryList.clear();
    // Read it a line at a time
    while (!in.eof()) {
        in.getline(line, sizeof(line));
        // Parse the line
        AutoTouple touple;
        // If we get something intelligble, add it to the list
        if (parseLine(line, &touple))
            directoryList.push_back(touple);
    }
    fb.close();
#ifdef DEBUG_AUTOLIST
    cout << directoryList.size() << " item(s) loaded" << endl;
#endif
}

int AutoList::parseLine(string line, AutoTouple *result) {
    // Split the line at the equal sign into name/value pair
    string::size_type equalPos = line.find('=');
    if (equalPos == string::npos) {
#ifdef DEBUG_AUTOLIST
        cout << "Equal not found" << endl;
#endif
        return 0;
    }
    string value = string(line, equalPos + 1);
    string name = string(line, 0, equalPos);
    // Attempt to extract a group name from the name (auto.{GROUP}.{NUMBER})
    string groupName;
    extractGroupName(name, groupName);
    // If it is a description, ignore (auto.{GROUP}.description)
    if (name.find(".description") == name.length() - strlen(".description")) {
#ifdef DEBUG_AUTOLIST
        cout << "Description, so skipping" << endl;
#endif
        return 0;
    }
#ifdef DEBUG_AUTOLIST
        cout << "Name: " << name << ", Value: " << value << ", Group: " << groupName << endl;
#endif
    // Create and return new object
    result->group = groupName;
    result->directory = value;
    return 1;
}

void AutoList::extractGroupName(string line, string &name) {
    string::size_type pos1 = line.find(".");
    string::size_type pos2 = line.find(".", pos1 + 1);
    if ((pos1 != string::npos) &&
        (pos2 != string::npos) &&
        (pos2 > pos1)) {
            name.clear();
            name.append(line, pos1+1, pos2 - pos1 - 1);
    }
}

list<AutoTouple>::const_iterator AutoList::begin() {
    return directoryList.begin();
}

list<AutoTouple>::const_iterator AutoList::end() {
    return directoryList.end();
}

void AutoList::dump() {
    list<AutoTouple>::const_iterator here = this->directoryList.begin();
    list<AutoTouple>::const_iterator last = this->directoryList.end();
    for(++here; here != last; ++here) {
        cout << "Group: " << here->group << " Directory: " << here->directory << endl;
    }
};

