#include <stdio.h>
#include <curl/curl.h>
#include "tryurl.h"
#include <string>
#include <list>
#include <iterator>
#include <iostream>
using namespace std;

//#define DEBUG_TRYURL

list <string> *TryUrl::DEFAULT_EXTENSION_LIST = buildExtensionList();

/**
 * Constructor--initialize cURL
 */
TryUrl::TryUrl() {
    curl_global_init(CURL_GLOBAL_ALL);
    curl = curl_easy_init();
    curl_easy_setopt(curl, CURLOPT_NOPROGRESS, 1);      // No progress bar
    curl_easy_setopt(curl, CURLOPT_USERAGENT, "wmap");  // Client string
    curl_easy_setopt(curl, CURLOPT_NOBODY, 1);          // HEAD request
}

/**
 * Destructor--clean up cURL
 */
TryUrl::~TryUrl() {
    curl_global_cleanup();
    curl_easy_cleanup(curl);
}

void TryUrl::tryUrl(string base, string extended, list<string> *extensions, list<string> *result) {
    // Sanity-check the input
    if ((base.length() < 7) || (extended.length() <= 0)) {
#ifdef DEBUG_TRYURL
        cout << "Base or extended were not long enough" << endl;
#endif
        return;
    }
    if (base.substr(0, 7).compare("http://")) {
#ifdef DEBUG_TRYURL
        cout << "Base didn't start with HTTP" << endl;
        cout << base.substr(0, 6) << endl;
#endif
        return;
    }
    // Ensure base ends with a slash
    if (base.c_str()[base.length() - 1] != '/')
        base.append("/");
    
    // First, try the base as a folder
    string urlResult;
    string testUrl = base + extended;
    // If it is a directory (i.e. on dot-something extension, append final slash)
    if (extended.find('.') == string::npos)
        testUrl.append("/");
    tryUrl(testUrl, &urlResult);
    result->push_back(urlResult);
    
    // Next, try all extensions (if any were given)
    list<string>::const_iterator i;
    if (extensions) {
        for (i = extensions->begin(); i != extensions->end(); i++) {
            urlResult.clear();
            tryUrl(base + extended + *i, &urlResult);
            result->push_back(urlResult);
        }
    }
    
    // Return the result
    return;
}

long TryUrl::tryUrl(string url, string *result) {
    CURLcode code;
    char errorStr[CURL_ERROR_SIZE] = "Unspecified error";
    long responseCode;
    
#ifdef DEBUG_TRYURL
    cout << "Trying " << url << endl;
#endif
    if (curl) {
        curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, errorStr);
        curl_easy_setopt(curl, CURLOPT_URL, url.c_str());
        //curl_easy_setopt(curl, CURLOPT_
        code = curl_easy_perform(curl);
        if (code != 0) {
            result->append("xxx cURL error: ");
            result->append("errorStr");            
            return 0;
        }
        code = curl_easy_getinfo(curl, CURLINFO_HTTP_CODE, &responseCode);
        if (code != 0) {
            result->append("xxx cURL could not extract response code");
            return 0;
        }else{
            errorStr[0] = 0;
            snprintf(errorStr, 4, "%3ld", responseCode);
            result->append(errorStr);
            result->append(" ");
            result->append(url);
            if ( ((responseCode >= 200) && (responseCode <= 299))
                 || (responseCode == 403))
                 result->append(" <<<<<<<");
            return responseCode;
        }
    }
    result->append("xxx cURL not initialized");
    return 0;
}

list<string> *TryUrl::buildExtensionList() {
    // This list never gets destructed, but that is okay because
    // it is for a private static variable
    list<string> *result = new list<string>;
    result->push_back(".html");
    result->push_back(".htm");
    result->push_back(".php");
    result->push_back(".php3");
    result->push_back(".phps");
    result->push_back(".asp");
    result->push_back(".pl");
    result->push_back(".jsp");
    result->push_back(".txt");
    result->push_back(".jpg");
    result->push_back(".gif");
    result->push_back(".png");
    return result;
}
