#ifndef TRY_URL_H
#define TRY_URL_H

#include <string>
#include <list>
#include <curl/curl.h>
using namespace std;

/**
 * Class for wrapping cURL, allowing you to simply pass in a URL (or set
 * thereof) and get an HTTP response code (or set thereof) without
 * worrying about cURL handles and all that.
 */
class TryUrl {
    public:
        /**
         * Static list of common extensions (useful for manual scanning
         */
        static list<string> *DEFAULT_EXTENSION_LIST;
        
        /**
         * Standard constructor
         */
        TryUrl();
        
        /**
         * Standard destructor
         */
        ~TryUrl();
        
        /**
         * Given a base URL, a base filename, and a list of extensions, search
         * for the file with each extension.  For instance, a base URL of
         * "http://netninja.com," a base filename of "monkey," and a list of
         * extensions {"html","php","asp"} will hit the following URLs:
         *  - http://netninja.com/monkey/ (no extension...a directory)
         *  - http://netninja.com/monkey.html
         *  - http://netninja.com/monkey.php
         *  - http://netninja.com/monkey.asp
         * Results are returned in an array of machine-parsable data--basically
         * an array of tryUrl(string, string*) answers.
         *
         * @param base base URL
         * @param extended base filename
         * @param extensions list of extensions to try
         * @param result results
         */
        void tryUrl(string base, string extended, list<string> *extensions, list<string> *result);
        
        /**
         * Attempt to request a URL from a remote host.  The result
         * contains a parsable string of the following format:
         * {three digit response code}{space}{URL}{optional "<<<<<<<" if found}
         *
         * The three digit response code could be "xxx" if there was in
         * internal error.
         *
         * @param url URL to try
         * @param result the result of the request
         * @return HTTP response code (or zero if error)
         */
        long tryUrl(string url, string *result);
        
    private:
        /**
         * cURL handle for resuse across the life of this object
         */
        CURL *curl;
        
        /**
         * Static initializer for the DEFAULT_EXTENSION_LIST member
         */
        static list<string> *buildExtensionList();
};

#endif
