#include <unistd.h>
#include <stdio.h>
#include <iostream>
#include "tryurl.h"
#include "wmap.h"

#define DEBUG_WMAP

const char WMap::folderCheckPaths[][64] = {
    "./",
    "/etc/wmap/",
    "/etc/",
    "/usr/share/wmap/",
    "/usr/share/",
    "/usr/local/etc/wmap/",
    "/usr/local/etc/",
    "/usr/local/share/wmap/",
    "/usr/local/share/"
    };
const char WMap::autoPropertiesFilename[] = "auto.properties";
            
WMap::WMap(string base) {
    this->base = base;
    this->runMode = RUN_AUTOMATIC;
    this->outputFunction = NULL;
    this->errorFunction = NULL;
    this->propertiesFilePath = "";
}

WMap::~WMap() {
}

int WMap::loadAutolist() {
    int found = 0;
    string filename;
    if (this->propertiesFilePath.length() == 0) {
        for (unsigned int i=0; i<sizeof(folderCheckPaths); i++) {
            filename = folderCheckPaths[i];
            filename.append(autoPropertiesFilename);
            FILE *f = fopen(filename.c_str(), "r");
            if (f != NULL) {
                fclose(f);
                found = 1;
                break;
            }
        }
    }else{
        filename = this->propertiesFilePath;
        FILE *f = fopen(filename.c_str(), "r");
        if (f != NULL) {
            fclose(f);
            found = 1;
        }
    }
    if (!found) {
        printError("Unable to find auto.properties");
        return 0;
    }
#ifdef DEBUG_WMAP
    cout << "Loading " << filename << endl;
#endif
    autoList.loadFile(filename);
    return 1;
}

void WMap::run() {
    if (this->runMode == RUN_AUTOMATIC)
        runAutomatic();
    else if (this->runMode == RUN_MANUAL)
        runManual();
}

void WMap::runAutomatic() {
    if (!loadAutolist())
        return;
    list<AutoTouple> found;
    list<AutoTouple> forbidden;
    list<string> results;
    if (base.c_str()[base.length() - 1] != '/')
        base.append("/");
    TryUrl tryUrl;
    for (list<AutoTouple>::const_iterator touple = autoList.begin(); touple != autoList.end(); touple++) {
        results.clear();
        tryUrl.tryUrl(base, touple->directory, (list<string> *) NULL, &results);
        if (results.size() > 0)
            printResult(*results.begin());
        for (list<string>::iterator result = results.begin(); result != results.end(); result++) {
            if (result->c_str()[0] == '2')
                found.push_back(*touple);
            if (result->substr(0, 3) == "403")
                forbidden.push_back(*touple);
        }
        doDelay();
    }
    printResult("Results:");
    for (list<AutoTouple>::iterator i = found.begin(); i != found.end(); i++) {
        cout << "EXISTS, LOADS         " << base << i->directory << endl;
    }
    for (list<AutoTouple>::iterator i = forbidden.begin(); i != forbidden.end(); i++) {
        cout << "EXISTS, NOT BROWSABLE " << base << i->directory << endl;
    }
}

void WMap::runManual() {
    TryUrl tryUrl;
    while (1) {
        //printResult("Enter directory/file name to search for at base URL (enter to quit)");
        string dir = this->inputFunction();
        if (dir.length() == 0)
            return;
        list<string> results;
        tryUrl.tryUrl(this->base, dir, TryUrl::DEFAULT_EXTENSION_LIST, &results);
        for (list<string>::iterator i = results.begin(); i != results.end(); i++)
            printResult(*i);
    }
}

void WMap::setRunMode(WMapRunModeType runMode) {
    this->runMode = runMode;
}

void WMap::setDelay(int seconds) {
    this->delay = seconds;
}

void WMap::setPropertiesFilePath(string filePath) {
    this->propertiesFilePath = filePath;
}

void WMap::printResult(string line) {
    this->outputFunction(line.c_str());
}

void WMap::printError(string line) {
    this->errorFunction(line.c_str());
}

void WMap::doDelay() {
    sleep(this->delay);
}

void WMap::setOutputFunction(WMAP_OUTPUT_FUNC) {
    this->outputFunction = outputFunction;
}

void WMap::setErrorFunction(WMAP_ERROR_FUNC) {
    this->errorFunction = errorFunction;
}

void WMap::setInputFunction(WMAP_INPUT_FUNC) {
    this->inputFunction = inputFunction;
}

