#ifndef _WMAP_H_
#define _WMAP_H_
#include <string>
#include "autolist.h"

#define WMAP_OUTPUT_FUNC void (*outputFunction)(const char *)
#define WMAP_ERROR_FUNC void (*errorFunction)(const char *)
#define WMAP_INPUT_FUNC string (*inputFunction)(void)

using namespace std;

typedef enum WMapRunMode {
    RUN_AUTOMATIC, RUN_MANUAL
} WMapRunModeType;

class WMap {
    public:

        /** Constructor that takes URL*/
        WMap(string base);
        
        /** Destructor */
        ~WMap();
        
        /** Run it! */
        void run();
        
        /** 
         * Set the run mode
         *@param type either RUN_AUTOMATIC or RUN_MANUAL
         */
        void setRunMode(WMapRunModeType runMode);
        
        /**
         * Set the delay between requests for automatic mode
         */
        void setDelay(int seconds);
        
        /**
         * Set a callback for normal output (typically, this would
         * just be a console printf type of function, but could
         * easily append text to a GUI text area, etc.
         */
        void setOutputFunction(WMAP_OUTPUT_FUNC);
        
        /**
         * Set a callback for error output (typically, this would
         * just be a console printf type of function, but could
         * easily append text to a GUI text area in an alternate
         * color, etc.
         */
        void setErrorFunction(WMAP_ERROR_FUNC);

        /** 
         * Define a function to retrieve a line of user input.
         */
        void setInputFunction(WMAP_INPUT_FUNC);
         
        /** 
         * Overrides the auto-search of the properties file to
         * the specific one given.
         */
         void setPropertiesFilePath(string filePath);

    private:
        static const char folderCheckPaths[][64];
        static const char autoPropertiesFilename[];
        /** Basic constructor hidden */
        WMap();
        /** Base URL */
        string base;
        /** Properties file override (empty string if no override) */
        string propertiesFilePath;
        /** Run mode */
        WMapRunModeType runMode;
        /** Number of seconds to delay between requests in automatic mode */
        int delay;
        /** Print a result to the screen */
        void printResult(string line);
        /** Print a result to the screen */
        void printError(string line);
        /** Perform the necessary delay */
        void doDelay();
        void runAutomatic();
        void runManual();
        /**
         * Load the autolist file
         * @return 0 for fail, 1 for success
         */
        int loadAutolist();
        WMAP_OUTPUT_FUNC;
        WMAP_ERROR_FUNC;
        WMAP_INPUT_FUNC;
        AutoList autoList;
};

#endif
